Package: biglasso
Title: Extending Lasso Model Fitting to Big Data
Version: 1.7.2
Authors@R: c(
  person("Yaohui", "Zeng", role = c("aut")),
  person("Chuyi", "Wang", role = c("aut")),
  person("Tabitha", "Peter", role = c("aut")),
  person("Patrick", "Breheny", role=c("aut","cre"),
    email="patrick-breheny@uiowa.edu", comment=c(ORCID="0000-0002-0650-1119"))
  )
Description: Extend lasso and elastic-net model fitting for large data sets that
  cannot be loaded into memory. Designed to be more memory- and
  computation-efficient than existing lasso-fitting packages like 'glmnet' and
  'ncvreg', thus allowing the user to analyze big data with limited RAM
  <doi:10.32614/RJ-2021-001>.
License: GPL-3
URL: https://pbreheny.github.io/biglasso/,
        https://github.com/pbreheny/biglasso
BugReports: https://github.com/pbreheny/biglasso/issues
Depends: R (>= 3.2.0), bigmemory (>= 4.5.0), Matrix, ncvreg
Imports: Rcpp (>= 0.12.1), methods
Suggests: glmnet, knitr, parallel, rmarkdown, survival, tinytest
LinkingTo: Rcpp, RcppArmadillo (>= 0.8.600), bigmemory, BH
VignetteBuilder: knitr
Encoding: UTF-8
Config/roxygen2/version: 8.1.0
NeedsCompilation: yes
Packaged: 2026-08-24 19:22:10 UTC; pbreheny
Author: Yaohui Zeng [aut],
  Chuyi Wang [aut],
  Tabitha Peter [aut],
  Patrick Breheny [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0650-1119>)
Maintainer: Patrick Breheny <patrick-breheny@uiowa.edu>
Repository: CRAN
Date/Publication: 2026-08-25 11:01:05 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-08-25 12:39:19 UTC; unix
Archs: biglasso.so.dSYM
