Package: biglasso
Version: 1.6.0
Date: 2024-04-21
Title: Extending Lasso Model Fitting to Big Data
Authors@R: c(
    person("Yaohui", "Zeng", role = c("aut")),
    person("Chuyi", "Wang", role = c("aut")),
    person("Tabitha", "Peter", role = c("aut")),
    person("Patrick", "Breheny", role=c("aut","cre"),
      email="patrick-breheny@uiowa.edu", comment=c(ORCID="0000-0002-0650-1119")))
Maintainer: Patrick Breheny <patrick-breheny@uiowa.edu>
Description: Extend lasso and elastic-net model fitting for ultra
    high-dimensional, multi-gigabyte data sets that cannot be loaded into
    memory. Designed to be more memory- and computation-efficient than existing
    lasso-fitting packages like 'glmnet' and 'ncvreg', thus allowing
    the user to analyze big data analysis even on an ordinary laptop.
License: GPL-3
URL: https://pbreheny.github.io/biglasso/index.html,
        https://github.com/pbreheny/biglasso,
        https://arxiv.org/abs/1701.05936
BugReports: https://github.com/pbreheny/biglasso/issues
Depends: R (>= 3.2.0), bigmemory (>= 4.5.0), Matrix, ncvreg
Imports: Rcpp (>= 0.12.1), methods
LinkingTo: Rcpp, RcppArmadillo (>= 0.8.600), bigmemory, BH
VignetteBuilder: knitr
Suggests: parallel, testthat, glmnet, survival, knitr, rmarkdown
RoxygenNote: 7.3.1
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-04-21 16:57:26 UTC; pbreheny
Author: Yaohui Zeng [aut],
  Chuyi Wang [aut],
  Tabitha Peter [aut],
  Patrick Breheny [aut, cre] (<https://orcid.org/0000-0002-0650-1119>)
Repository: CRAN
Date/Publication: 2024-04-21 17:12:36 UTC
Built: R 4.2.3; aarch64-apple-darwin20; 2024-04-21 18:08:28 UTC; unix
Archs: biglasso.so.dSYM
