Package: biglasso
Version: 1.6.1
Date: 2025-03-05
Title: Extending Lasso Model Fitting to Big Data
Authors@R: c(
  person("Yaohui", "Zeng", role = c("aut")),
  person("Chuyi", "Wang", role = c("aut")),
  person("Tabitha", "Peter", role = c("aut")),
  person("Patrick", "Breheny", role=c("aut","cre"),
    email="patrick-breheny@uiowa.edu", comment=c(ORCID="0000-0002-0650-1119")))
Maintainer: Patrick Breheny <patrick-breheny@uiowa.edu>
Description: Extend lasso and elastic-net model fitting for large data sets that
  cannot be loaded into memory. Designed to be more memory- and
  computation-efficient than existing lasso-fitting packages like 'glmnet' and
  'ncvreg', thus allowing the user to analyze big data with limited RAM
  <doi:10.32614/RJ-2021-001>.
License: GPL-3
URL: https://pbreheny.github.io/biglasso/,
        https://github.com/pbreheny/biglasso
BugReports: https://github.com/pbreheny/biglasso/issues
Depends: R (>= 3.2.0), bigmemory (>= 4.5.0), Matrix, ncvreg
Imports: Rcpp (>= 0.12.1), methods
LinkingTo: Rcpp, RcppArmadillo (>= 0.8.600), bigmemory, BH
VignetteBuilder: knitr
Suggests: glmnet, knitr, parallel, rmarkdown, survival, tinytest
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-03-05 17:32:37 UTC; pbreheny
Author: Yaohui Zeng [aut],
  Chuyi Wang [aut],
  Tabitha Peter [aut],
  Patrick Breheny [aut, cre] (<https://orcid.org/0000-0002-0650-1119>)
Repository: CRAN
Date/Publication: 2025-03-05 18:10:14 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:56:08 UTC; unix
Archs: biglasso.so.dSYM
