# Generated by roxygen2: do not edit by hand

S3method(coef,biglasso)
S3method(coef,cv.biglasso)
S3method(coef,mbiglasso)
S3method(plot,biglasso)
S3method(plot,cv.biglasso)
S3method(plot,mbiglasso)
S3method(predict,biglasso)
S3method(predict,cv.biglasso)
S3method(predict,mbiglasso)
S3method(print,summary.cv.biglasso)
S3method(summary,cv.biglasso)
export(biglasso)
export(biglasso_fit)
export(biglasso_path)
export(cv.biglasso)
export(setupX)
import(bigmemory)
import(grDevices)
import(graphics)
import(ncvreg)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(methods,as)
importMethodsFrom(Matrix,crossprod)
useDynLib(biglasso, .registration = TRUE)
